@@ -608,6 +608,30 @@ xwl_window_rootful_set_app_id(struct xwl_window *xwl_window)
         xdg_toplevel_set_app_id(xwl_window->xdg_toplevel, app_id);
 }
 
+static void
+xwl_window_maybe_resize(struct xwl_window *xwl_window, int width, int height)
+{
+    struct xwl_screen *xwl_screen = xwl_window->xwl_screen;
+    struct xwl_output *xwl_output;
+    RRModePtr mode;
+
+    /* Clamp the size */
+    width = min(max(width, MIN_ROOTFUL_WIDTH), MAX_ROOTFUL_WIDTH);
+    height = min(max(height, MIN_ROOTFUL_HEIGHT), MAX_ROOTFUL_HEIGHT);
+
+    if (width == xwl_screen->width && height == xwl_screen->height)
+        return;
+
+    xwl_output = xwl_screen_get_fixed_or_first_output(xwl_screen);
+    if (!xwl_randr_add_modes_fixed(xwl_output, width, height))
+        return;
+
+    mode = xwl_output_find_mode(xwl_output, width, height);
+    xwl_output_set_mode_fixed(xwl_output, mode);
+
+    xwl_window_attach_buffer(xwl_window);
+}
+
 #ifdef XWL_HAS_LIBDECOR
 static void
 xwl_window_libdecor_set_size_limits(struct xwl_window *xwl_window)
@@ -632,24 +656,6 @@ xwl_window_update_libdecor_size(struct xwl_window *xwl_window,
     }
 }
 
-static void
-xwl_window_libdecor_resize(struct xwl_window *xwl_window, int width, int height)
-{
-    struct xwl_screen *xwl_screen = xwl_window->xwl_screen;
-    struct xwl_output *xwl_output;
-    RRModePtr mode;
-
-    xwl_output = xwl_screen_get_fixed_or_first_output(xwl_screen);
-    if (!xwl_randr_add_modes_fixed(xwl_output, width, height))
-        return;
-
-    mode = xwl_output_find_mode(xwl_output, width, height);
-    xwl_output_set_mode_fixed(xwl_output, mode);
-
-    xwl_window_attach_buffer(xwl_window);
-    wl_surface_commit(xwl_window->surface);
-}
-
 static void
 handle_libdecor_configure(struct libdecor_frame *frame,
                           struct libdecor_configuration *configuration,
@@ -663,15 +669,11 @@ handle_libdecor_configure(struct libdecor_frame *frame,
         width = xwl_screen->width;
         height = xwl_screen->height;
     }
-    /* Clamp the size */
-    width = min(max(width, MIN_ROOTFUL_WIDTH), MAX_ROOTFUL_WIDTH);
-    height = min(max(height, MIN_ROOTFUL_HEIGHT), MAX_ROOTFUL_HEIGHT) ;
-
-    if (xwl_screen->width != width || xwl_screen->height != height)
-        xwl_window_libdecor_resize(xwl_window, width, height);
 
+    xwl_window_maybe_resize(xwl_window, width, height);
     xwl_window_update_libdecor_size(xwl_window, configuration,
                                     xwl_screen->width, xwl_screen->height);
+    wl_surface_commit(xwl_window->surface);
 }
 
 static void
@@ -763,6 +765,14 @@ xdg_toplevel_handle_configure(void *data,
                               int32_t height,
                               struct wl_array *states)
 {
+    struct xwl_window *xwl_window = data;
+
+    /* Maintain our current size if no dimensions are requested */
+    if (width == 0 && height == 0)
+        return;
+
+    /* This will be committed by the xdg_surface.configure handler */
+    xwl_window_maybe_resize(xwl_window, width, height);
 }
 
 static void
@@ -821,7 +831,7 @@ xwl_create_root_surface(struct xwl_window *xwl_window)
 
         xdg_toplevel_add_listener(xwl_window->xdg_toplevel,
                                   &xdg_toplevel_listener,
-                                  NULL);
+                                  xwl_window);
     }
 
     xwl_window_rootful_update_title(xwl_window);
